var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { deserializeError } from 'serialize-error';
/** Add custom cypress command to compare image snapshots of an element or the window. */
function addCompareSnapshotCommand(screenshotOptions) {
    Cypress.Commands.add('compareSnapshot', { prevSubject: 'optional' }, function (subject, name, commandOptions) {
        if (name === undefined || name === '') {
            throw new Error('Snapshot name must be specified');
        }
        // prepare screenshot options
        var errorThreshold = 0;
        if (typeof commandOptions === 'object') {
            screenshotOptions = __assign(__assign({}, screenshotOptions), commandOptions);
        }
        if (typeof commandOptions === 'number') {
            errorThreshold = commandOptions;
        }
        var visualRegressionOptions = prepareOptions(name, errorThreshold, screenshotOptions);
        return takeScreenshot(subject, name, screenshotOptions).then(function (screenshotAbsolutePath) {
            visualRegressionOptions.screenshotAbsolutePath = screenshotAbsolutePath;
            switch (visualRegressionOptions.type) {
                case 'regression':
                    return compareScreenshots(visualRegressionOptions);
                case 'base':
                    return cy.task('updateSnapshot', visualRegressionOptions);
                default:
                    throw new Error("The \"type\" environment variable is unknown. \nExpected: \"regression\" or \"base\" \nActual: ".concat(visualRegressionOptions.type));
            }
        });
    });
}
function prepareOptions(name, errorThreshold, screenshotOptions) {
    var _a, _b, _c, _d;
    var options = {
        type: Cypress.env('visualRegression').type,
        screenshotName: name,
        specName: Cypress.spec.name,
        screenshotAbsolutePath: 'null',
        errorThreshold: errorThreshold,
        baseDirectory: (_a = Cypress.env('visualRegression')) === null || _a === void 0 ? void 0 : _a.baseDirectory,
        diffDirectory: (_b = Cypress.env('visualRegression')) === null || _b === void 0 ? void 0 : _b.diffDirectory,
        generateDiff: (_c = Cypress.env('visualRegression')) === null || _c === void 0 ? void 0 : _c.generateDiff,
        failSilently: (_d = Cypress.env('visualRegression')) === null || _d === void 0 ? void 0 : _d.failSilently
    };
    if ((screenshotOptions === null || screenshotOptions === void 0 ? void 0 : screenshotOptions.failSilently) !== undefined) {
        options.failSilently = screenshotOptions.failSilently;
    }
    else if (Cypress.env('visualRegression').failSilently !== undefined) {
        options.failSilently = Cypress.env('visualRegression').failSilently;
    }
    // deprecation methods
    if (Cypress.env('type') !== undefined) {
        console.error("Environment variable 'type' is deprecated. Please check README.md file for latest configuration.");
        options.type = Cypress.env('type');
    }
    if (Cypress.env('failSilently') !== undefined) {
        console.error("Environment variable 'failSilently' is deprecated. Please check README.md file for latest configuration.");
    }
    if (Cypress.env('SNAPSHOT_BASE_DIRECTORY') !== undefined) {
        console.error("Environment variable 'SNAPSHOT_BASE_DIRECTORY' is deprecated. Please check README.md file for latest configuration.");
        options.baseDirectory = Cypress.env('SNAPSHOT_BASE_DIRECTORY');
    }
    if (Cypress.env('SNAPSHOT_DIFF_DIRECTORY') !== undefined) {
        console.error("Environment variable 'SNAPSHOT_DIFF_DIRECTORY' is deprecated. Please check README.md file for latest configuration.");
        options.diffDirectory = Cypress.env('SNAPSHOT_DIFF_DIRECTORY');
    }
    if (Cypress.env('INTEGRATION_FOLDER') !== undefined) {
        console.error("Environment variable 'INTEGRATION_FOLDER' is deprecated. Please check README.md file for latest configuration.");
    }
    if (Cypress.env('ALWAYS_GENERATE_DIFF') !== undefined) {
        console.error("Environment variable 'ALWAYS_GENERATE_DIFF' is deprecated. Please check README.md file for latest configuration.");
        // eslint-disable-next-line @typescript-eslint/strict-boolean-expressions
        options.generateDiff = Cypress.env('ALWAYS_GENERATE_DIFF') ? 'always' : 'never';
    }
    if (Cypress.env('ALLOW_VISUAL_REGRESSION_TO_FAIL') !== undefined) {
        console.error("Environment variable 'ALLOW_VISUAL_REGRESSION_TO_FAIL' is deprecated. Please check README.md file for latest configuration.");
        options.failSilently = Cypress.env('ALLOW_VISUAL_REGRESSION_TO_FAIL');
    }
    return options;
}
/** Take a screenshot and move screenshot to base or actual folder */
function takeScreenshot(subject, name, screenshotOptions) {
    var objToOperateOn = subject !== undefined ? cy.get(subject) : cy;
    var screenshotPath;
    return (objToOperateOn
        .screenshot(name, __assign(__assign({}, screenshotOptions), { 
        // eslint-disable-next-line
        onAfterScreenshot: function (_el, props) {
            screenshotPath = props.path;
        } }))
        // @ts-ignore
        .then(function () {
        return screenshotPath;
    }));
}
/** Call the plugin to compare snapshot images and generate a diff */
function compareScreenshots(options) {
    return cy.task('compareSnapshots', options).then(function (results) {
        if (results.error !== undefined && !options.failSilently) {
            throw deserializeError(results.error);
        }
        return results;
    });
}
export default addCompareSnapshotCommand;
