var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { createWriteStream, promises as fs } from 'fs';
import * as path from 'path';
import pixelMatch from 'pixelmatch';
import { PNG } from 'pngjs';
import sanitize from 'sanitize-filename';
import { serializeError } from 'serialize-error';
import { adjustCanvas, parseImage } from './utils/image';
import { logger } from './utils/logger';
/**
 * Update the base snapshot .png by copying the generated snapshot to the base snapshot directory.
 * The target path is constructed from parts at runtime in node to be OS independent.
 * */
var updateSnapshot = function (options) { return __awaiter(void 0, void 0, void 0, function () {
    var toDir, destDir, destFile;
    var _a;
    return __generator(this, function (_b) {
        switch (_b.label) {
            case 0:
                toDir = (_a = options.baseDirectory) !== null && _a !== void 0 ? _a : path.join(process.cwd(), 'cypress', 'snapshots', 'base');
                destDir = path.join(toDir, options.specName);
                destFile = path.join(destDir, "".concat(options.screenshotName, ".png"));
                return [4 /*yield*/, fs.mkdir(destDir, { recursive: true })];
            case 1:
                _b.sent();
                return [4 /*yield*/, fs.copyFile(options.screenshotAbsolutePath, destFile)];
            case 2:
                _b.sent();
                logger.debug('Updated base snapshot "%s" at "%s"', options.screenshotName, destFile);
                return [2 /*return*/, { baseGenerated: true }];
        }
    });
}); };
/**
 * Cypress plugin to compare image snapshots & generate a diff image.
 * Uses the pixelmatch library internally.
 * */
var compareSnapshots = function (options) { return __awaiter(void 0, void 0, void 0, function () {
    var snapshotBaseDirectory, snapshotDiffDirectory, fileName, specFolder, actualImage, expectedImage, diffImage, _a, imgExpected, imgActual, diffPNG, imgActualFullCanvas, imgExpectedFullCanvas, mismatchedPixels, percentage;
    var _b, _c;
    return __generator(this, function (_d) {
        switch (_d.label) {
            case 0:
                snapshotBaseDirectory = (_b = options.baseDirectory) !== null && _b !== void 0 ? _b : path.join(process.cwd(), 'cypress', 'snapshots', 'base');
                snapshotDiffDirectory = (_c = options.diffDirectory) !== null && _c !== void 0 ? _c : path.join(process.cwd(), 'cypress', 'snapshots', 'diff');
                fileName = sanitize(options.screenshotName);
                specFolder = path.join(snapshotDiffDirectory, options.specName);
                actualImage = options.screenshotAbsolutePath;
                expectedImage = path.join(snapshotBaseDirectory, options.specName, "".concat(fileName, ".png"));
                diffImage = path.join(snapshotDiffDirectory, options.specName, "".concat(fileName, ".png"));
                return [4 /*yield*/, Promise.all([parseImage(expectedImage), parseImage(actualImage)])];
            case 1:
                _a = _d.sent(), imgExpected = _a[0], imgActual = _a[1];
                diffPNG = new PNG({
                    width: Math.max(imgActual.width, imgExpected.width),
                    height: Math.max(imgActual.height, imgExpected.height)
                });
                imgActualFullCanvas = adjustCanvas(imgActual, diffPNG.width, diffPNG.height);
                imgExpectedFullCanvas = adjustCanvas(imgExpected, diffPNG.width, diffPNG.height);
                mismatchedPixels = pixelMatch(imgActualFullCanvas.data, imgExpectedFullCanvas.data, diffPNG.data, diffPNG.width, diffPNG.height, { threshold: 0.1 });
                percentage = Math.pow((mismatchedPixels / diffPNG.width / diffPNG.height), 0.5);
                if (!(percentage > options.errorThreshold)) return [3 /*break*/, 4];
                logger.error('Error in visual regression found: "%s"', percentage.toFixed(2));
                if (!(options.generateDiff !== 'never')) return [3 /*break*/, 3];
                return [4 /*yield*/, generateImage(diffPNG, diffImage, specFolder)];
            case 2:
                _d.sent();
                _d.label = 3;
            case 3: return [2 /*return*/, {
                    error: serializeError(new Error("The \"".concat(fileName, "\" image is different. Threshold limit exceeded!\n          Expected: ").concat(options.errorThreshold, "\n          Actual: ").concat(percentage))),
                    mismatchedPixels: mismatchedPixels,
                    percentage: percentage
                }];
            case 4:
                if (!(options.generateDiff === 'always')) return [3 /*break*/, 6];
                return [4 /*yield*/, generateImage(diffPNG, diffImage, specFolder)];
            case 5:
                _d.sent();
                _d.label = 6;
            case 6: return [2 /*return*/, {
                    mismatchedPixels: mismatchedPixels,
                    percentage: percentage
                }];
        }
    });
}); };
/** Configure the plugin to compare snapshots. */
var configureVisualRegression = function (on) {
    on('task', {
        compareSnapshots: compareSnapshots,
        updateSnapshot: updateSnapshot
    });
};
function generateImage(diffPNG, image, path) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, fs.mkdir(path, { recursive: true })];
                case 1:
                    _a.sent();
                    diffPNG.pack().pipe(createWriteStream(image));
                    logger.debug('Image with pixel difference generated: "%d"', image);
                    return [2 /*return*/];
            }
        });
    });
}
export default configureVisualRegression;
