var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { createReadStream } from 'fs';
import { PNG } from 'pngjs';
import { logger } from './logger';
/**
 * Parses an image file and returns a Promise that resolves with a PNG instance.
 *
 * @param {string} imagePath - The path to the image file to parse.
 * @returns {Promise<PNG>} A Promise that resolves with a PNG instance representing the parsed image.
 * @throws {Error} Throws an error if the specified image file does not exist or if there was an error parsing the file.
 */
export var parseImage = function (imagePath) { return __awaiter(void 0, void 0, void 0, function () {
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, new Promise(function (resolve, reject) {
                    var stream = createReadStream(imagePath);
                    stream.on('error', function (error) {
                        logger.error('Failed to open "%s" with message: "%s"', imagePath, error.message);
                        reject(new Error("File '".concat(imagePath, "' does not exist.")));
                    });
                    stream
                        .pipe(new PNG())
                        .on('parsed', function () {
                        resolve(this);
                    })
                        .on('error', function (error) {
                        logger.error('Failed to parse image "%s" with message: "%s"', imagePath, error.message);
                        reject(error);
                    });
                })];
            case 1: return [2 /*return*/, _a.sent()];
        }
    });
}); };
/**
 * Adjusts the canvas size of an image.
 *
 * @param {PNG} image - The input image.
 * @param {number} width - The target width of the image.
 * @param {number} height - The target height of the image.
 *
 * @returns {PNG} The new image with adjusted canvas size.
 */
export var adjustCanvas = function (image, width, height) {
    if (image.width === width && image.height === height) {
        return image;
    }
    var imageAdjustedCanvas = new PNG({ width: width, height: height, inputHasAlpha: true });
    PNG.bitblt(image, imageAdjustedCanvas, 0, 0, image.width, image.height, 0, 0);
    return imageAdjustedCanvas;
};
